/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup arp
 *  @{
 */

/** @file
 */

#ifndef __NET_ARP_H__
#define __NET_ARP_H__

#include <ipc/ipc.h>

#include "../../netif/device.h"

#include "../../structures/generic_char_map.h"
#include "../../structures/int_map.h"
#include "../../structures/measured_strings.h"

typedef struct arp_globals	arp_globals_t;
typedef arp_globals_t *		arp_globals_ref;
typedef struct arp_device	arp_device_t;
typedef arp_device_t *		arp_device_ref;
typedef struct arp_proto	arp_proto_t;
typedef arp_proto_t *		arp_proto_ref;

DEVICE_MAP_DECLARE( arp_cache, arp_device_t )

INT_MAP_DECLARE( arp_protos, arp_proto_t )

GENERIC_CHAR_MAP_DECLARE( arp_addr, measured_string_t )

struct arp_device{
	device_id_t			device_id;
	ipcarg_t			hardware;
	ipcarg_t			protocol;
	ipcarg_t			prefix;
	ipcarg_t			content;
	ipcarg_t			sufix;
	measured_string_ref	addr;
	char *				addr_data;
	measured_string_ref	broadcast_addr;
	char *				broadcast_data;
	services_t			service;
	int					phone;
	arp_protos_t		protos;
};

struct arp_proto{
	services_t			service;
//	int					phone;
	measured_string_ref	addr;
	char *				addr_data;
	arp_addr_t			addresses;
};

struct	arp_globals{
	int			networking_phone;
	arp_cache_t	cache;
};

#endif

/** @}
 */
