/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 * @{
 */

/** @file
 */

#ifndef __NET_PACKET_H__
#define __NET_PACKET_H__

#define PACKET_PREPEND( packet, type )		( type * ) packet_prepend(( packet ), sizeof( type ))
#define PACKET_APPEND( packet, type )		( type * ) packet_append(( packet ), sizeof( type ))
#define PACKET_TRIM( packet, prefix, sufix )	packet_trim(( packet ), sizeof( prefix ), sizeof( sufix ))

typedef struct packet *	packet_t;
typedef packet_t *	packet_ref;

packet_t	packet_create( size_t max_prefix, size_t max_content, size_t max_sufix );
void *		packet_prepend( packet_t packet, size_t length );
void *		packet_append( packet_t packet, size_t length );
packet_t	packet_copy( packet_t packet );
int		packet_copy_data( packet_t packet, void * data, size_t length );
// TODO protocol identification?
int		packet_send( packet_t packet, int phone );
int		packet_receive( packet_ref packet );
int		packet_trim( packet_t packet, size_t prefix, size_t sufix );
int		packet_destroy( packet_t packet );
size_t		packet_get_data_length( packet_t packet );
void *		packet_get_data( packet_t packet );

#endif

/** @}
 */
